from feapder.db.mysqldb import MysqlDB


db = MysqlDB(
    ip="localhost", port=3306, db="feapder", user_name="feapder", user_pass="feapder123"
)

MysqlDB.from_url("mysql://feapder:feapder123@localhost:3306/feapder?charset=utf8mb4")


# -*- coding: utf-8 -*-
"""
Created on 2022/11/18 11:33 AM
---------
@summary:
---------
@author: Boris
@email: boris_liu@foxmail.com
"""
import feapder.setting as setting
from feapder.buffer.item_buffer import ItemBuffer
from feapder.db.redisdb import RedisDB
from feapder.network.item import Item, UpdateItem
from feapder.utils.log import log


class HandleFailedItems:
    def __init__(self, redis_key, task_table=None, item_buffer=None):
        if redis_key.endswith(":s_failed_items"):
            redis_key = redis_key.replace(":s_failed_items", "")

        self._redisdb = RedisDB()
        self._item_buffer = item_buffer or ItemBuffer(redis_key, task_table=task_table)

        self._table_failed_items = setting.TAB_FAILED_ITEMS.format(redis_key=redis_key)

    def get_failed_items(self, count=1):
        failed_items = self._redisdb.sget(
            self._table_failed_items, count=count, is_pop=False
        )
        return failed_items

    def reput_failed_items_to_db(self):
        log.debug("正在重新写入失败的items...")
        total_count = 0
        while True:
            try:
                failed_items = self.get_failed_items()
                if not failed_items:
                    break

                for data_str in failed_items:
                    data = eval(data_str)

                    for add in data.get("add"):
                        table = add.get("table")
                        datas = add.get("datas")
                        for _data in datas:
                            item = Item(**_data)
                            item.table_name = table
                            self._item_buffer.put_item(item)
                            total_count += 1

                    for update in data.get("update"):
                        table = update.get("table")
                        datas = update.get("datas")
                        update_keys = update.get("update_keys")
                        for _data in datas:
                            item = UpdateItem(**_data)
                            item.table_name = table
                            item.update_keys = update_keys
                            self._item_buffer.put_item(item)
                            total_count += 1

                    # 入库成功后删除
                    def delete_item():
                        self._redisdb.srem(self._table_failed_items, data_str)

                    self._item_buffer.put_item(delete_item)
                    self._item_buffer.flush()

            except Exception as e:
                log.exception(e)

        if total_count:
            log.debug("导入%s条失败item到数库" % total_count)
        else:
            log.debug("没有失败的item")

    def close(self):
        self._item_buffer.close()



# -*- coding: utf-8 -*-
"""
Created on 2022-06-10 14:30:54
---------
@summary:
---------
@author: Boris
"""

import feapder
from feapder import ArgumentParser


class TestTaskSpider(feapder.TaskSpider):
    def add_task(self):
        # 加种子任务 框架会调用这个函数，方便往redis里塞任务，但不能写成死循环。实际业务中可以自己写个脚本往redis里塞任务
        self._redisdb.zadd(self._task_table, {"id": 1, "url": "https://www.baidu.com"})

    def start_requests(self, task):
        task_id, url = task
        yield feapder.Request(url, task_id=task_id)

    def parse(self, request, response):
        # 提取网站title
        print(response.xpath("//title/text()").extract_first())
        # 提取网站描述
        print(response.xpath("//meta[@name='description']/@content").extract_first())
        print("网站地址: ", response.url)

        # mysql 需要更新任务状态为做完 即 state=1
        # yield self.update_task_batch(request.task_id)


def start(args):
    """
    用mysql做种子表
    """
    spider = TestTaskSpider(
        task_table="spider_task",
        task_keys=["id", "url"],
        redis_key="test:task_spider",
        keep_alive=True,
    )
    if args == 1:
        spider.start_monitor_task()
    else:
        spider.start()


def start2(args):
    """
    用redis做种子表
    """
    spider = TestTaskSpider(
        task_table="spider_task2",
        task_table_type="redis",
        redis_key="test:task_spider",
        keep_alive=True,
        use_mysql=False,
    )
    if args == 1:
        spider.start_monitor_task()
    else:
        spider.start()


if __name__ == "__main__":
    parser = ArgumentParser(description="测试TaskSpider")

    parser.add_argument(
        "--start", type=int, nargs=1, help="用mysql做种子表 (1|2）", function=start
    )
    parser.add_argument(
        "--start2", type=int, nargs=1, help="用redis做种子表 (1|2）", function=start2
    )

    parser.start()

    # 下发任务  python3 test_task_spider.py --start 1
    # 采集  python3 test_task_spider.py --start 2



# -*- coding: utf-8 -*-
"""
Created on 2021-03-30 10:27:21
---------
@summary:
---------
@author: Boris
"""

import feapder
from feapder import Item


class TestAirSpiderItem(feapder.AirSpider):
    __custom_setting__ = dict(
        MYSQL_IP="localhost",
        MYSQL_PORT=3306,
        MYSQL_DB="feapder",
        MYSQL_USER_NAME="feapder",
        MYSQL_USER_PASS="feapder123",
        ITEM_FILTER_ENABLE=True,  # item 去重
        ITEM_FILTER_SETTING = dict(
            filter_type=4  # 永久去重（BloomFilter） = 1 、内存去重（MemoryFilter） = 2、 临时去重（ExpireFilter）= 3、轻量去重（LiteFilter）= 4
        )
    )

    def start_requests(self):
        yield feapder.Request("https://www.baidu.com")

    def parse(self, request, response):
        title = response.xpath("string(//title)").extract_first()
        for i in range(3):
            item = Item()
            item.table_name = "spider_data"
            item.url = request.url
            item.title = title
            yield item


if __name__ == "__main__":
    TestAirSpiderItem().start()



__all__ = [
    "GuestUserPool",
    "GuestUser",
    "NormalUserPool",
    "NormalUser",
    "GoldUserPool",
    "GoldUser",
    "GoldUserStatus",
]

from .gold_user_pool import GoldUserPool, GoldUser, GoldUserStatus
from .guest_user_pool import GuestUserPool, GuestUser
from .normal_user_pool import NormalUserPool, NormalUser



# -*- coding: utf-8 -*-
"""
Created on 2022/9/21 11:17 AM
---------
@summary:
---------
@author: Boris
@email: boris_liu@foxmail.com
"""
import abc
from typing import List, Union


class BaseFilter:
    @abc.abstractmethod
    def add(
        self, keys: Union[List[str], str], *args, **kwargs
    ) -> Union[List[bool], bool]:
        """

        Args:
            keys: list / 单个值
            *args:
            **kwargs:

        Returns:
            list / 单个值 (如果数据已存在 返回 0 否则返回 1, 可以理解为是否添加成功)
        """
        pass

    @abc.abstractmethod
    def get(self, keys: Union[List[str], str]) -> Union[List[bool], bool]:
        """
        检查数据是否存在
        Args:
            keys: list / 单个值

        Returns:
            list / 单个值 (如果数据已存在 返回 1 否则返回 0)
        """
        pass



# -*- coding: utf-8 -*-
"""
Created on 2022-08-04 17:58:45
---------
@summary:
---------
@author: Boris
"""

import feapder
from feapder import ArgumentParser


class TestSpider(feapder.TaskSpider):
    # 自定义数据库，若项目中有setting.py文件，此自定义可删除
    __custom_setting__ = dict(
        REDISDB_IP_PORTS="localhost:6379",
        REDISDB_USER_PASS="",
        REDISDB_DB=0,
        MYSQL_IP="localhost",
        MYSQL_PORT=3306,
        MYSQL_DB="",
        MYSQL_USER_NAME="",
        MYSQL_USER_PASS="",
    )

    def start_requests(self, task):
        task_id = task.id
        url = task.url
        yield feapder.Request(url, task_id=task_id)

    def parse(self, request, response):
        # 提取网站title
        print(response.xpath("//title/text()").extract_first())
        # 提取网站描述
        print(response.xpath("//meta[@name='description']/@content").extract_first())
        print("网站地址: ", response.url)

        # mysql 需要更新任务状态为做完 即 state=1
        yield self.update_task_batch(request.task_id)


if __name__ == "__main__":
    # 用mysql做任务表，需要先建好任务任务表
    spider = TestSpider(
        redis_key="xxx:xxx",  # 分布式爬虫调度信息存储位置
        task_table="",  # mysql中的任务表
        task_keys=["id", "url"],  # 需要获取任务表里的字段名，可添加多个
        task_state="state",  # mysql中任务状态字段
    )

    # 用redis做任务表
    # spider = TestSpider(
    #     redis_key="xxx:xxxx",  # 分布式爬虫调度信息存储位置
    #     task_table="", # 任务表名
    #     task_table_type="redis", # 任务表类型为redis
    # )

    parser = ArgumentParser(description="TestSpider爬虫")

    parser.add_argument(
        "--start_master",
        action="store_true",
        help="添加任务",
        function=spider.start_monitor_task,
    )
    parser.add_argument(
        "--start_worker", action="store_true", help="启动爬虫", function=spider.start
    )

    parser.start()

    # 直接启动
    # spider.start()  # 启动爬虫
    # spider.start_monitor_task() # 添加任务

    # 通过命令行启动
    # python test_spider.py --start_master  # 添加任务
    # python test_spider.py --start_worker  # 启动爬虫


# -*- coding: utf-8 -*-
"""
Created on 2018-08-28 17:38:43
---------
@summary: 字符串转json
---------
@author: Boris
@email:  boris_liu@foxmail.com
"""

import pyperclip

import feapder.utils.tools as tools


class CreateJson:
    def get_data(self):
        """
        @summary: 从控制台读取多行
        ---------
        ---------
        @result:
        """
        input("请复制需要转换的内容（xxx:xxx格式，支持多行），复制后按任意键读取剪切板内容\n")

        text = pyperclip.paste()
        print(text + "\n")

        data = []
        for line in text.split("\n"):
            line = line.strip().replace("\t", " " * 4)
            if not line:
                break

            data.append(line)

        return data

    def create(self, sort_keys=False):
        contents = self.get_data()

        json = {}
        for content in contents:
            content = content.strip()
            if not content or content.startswith(":"):
                continue

            regex = "([^:\s]*)[:|\s]*(.*)"

            result = tools.get_info(content, regex, fetch_one=True)
            if result[0] in json:
                json[result[0]] = json[result[0]] + "&" + result[1]
            else:
                json[result[0]] = result[1].strip()

        print(tools.dumps_json(json, sort_keys=sort_keys))



# -*- coding: utf-8 -*-
"""
Created on 2021-02-08 16:06:12
---------
@summary:
---------
@author: Boris
"""

import feapder
from feapder import Item, UpdateItem


class TestMongo(feapder.AirSpider):
    __custom_setting__ = dict(
        ITEM_PIPELINES=["feapder.pipelines.mongo_pipeline.MongoPipeline"],
        MONGO_IP="localhost",
        MONGO_PORT=27017,
        MONGO_DB="feapder",
        MONGO_USER_NAME="",
        MONGO_USER_PASS="",
    )

    def start_requests(self):
        yield feapder.Request("https://www.baidu.com")

    def parse(self, request, response):
        title = response.xpath("//title/text()").extract_first()  # 取标题
        for i in range(10):
            item = Item()  # 声明一个item
            item.table_name = "test_mongo"
            item.title = title + str(666)  # 给item属性赋值
            item.i = i + 5
            item.c = "777"
            yield item  # 返回item， item会自动批量入库


if __name__ == "__main__":
    TestMongo().start()



from feapder.utils import metrics

# 初始化打点系统
metrics.init(
    influxdb_host="localhost",
    influxdb_port="8086",
    influxdb_udp_port="8089",
    influxdb_database="feapder",
    influxdb_user="***",
    influxdb_password="***",
    influxdb_measurement="test_metrics",
    debug=True,
)


for i in range(1000):
    metrics.emit_counter("total count", count=1000, classify="test5")
    for j in range(1000):
        metrics.emit_counter("key", count=1, classify="test5")

metrics.close()



# -*- coding: utf-8 -*-
"""
Created on 2022/9/21 11:28 AM
---------
@summary:
---------
@author: Boris
@email: boris_liu@foxmail.com
"""
from typing import List, Union, Set

from feapder.dedup.basefilter import BaseFilter


class LiteFilter(BaseFilter):
    def __init__(self):
        self.datas: Set[str] = set()

    def add(
        self, keys: Union[List[str], str], *args, **kwargs
    ) -> Union[List[int], int]:
        """

        Args:
            keys: list / 单个值
            *args:
            **kwargs:

        Returns:
            list / 单个值 (如果数据已存在 返回 0 否则返回 1, 可以理解为是否添加成功)
        """
        if isinstance(keys, list):
            is_add = []
            for key in keys:
                if key not in self.datas:
                    self.datas.add(key)
                    is_add.append(1)
                else:
                    is_add.append(0)
        else:
            if keys not in self.datas:
                is_add = 1
                self.datas.add(keys)
            else:
                is_add = 0
        return is_add

    def get(self, keys: Union[List[str], str]) -> Union[List[int], int]:
        """
        检查数据是否存在
        Args:
            keys: list / 单个值

        Returns:
            list / 单个值 (如果数据已存在 返回 1 否则返回 0)
        """
        if isinstance(keys, list):
            temp_set = set()
            is_exist = []
            for key in keys:
                # 数据本身重复或者数据在去重库里
                if key in temp_set or key in self.datas:
                    is_exist.append(1)
                else:
                    is_exist.append(0)
                    temp_set.add(key)

            return is_exist
        else:
            return int(keys in self.datas)
